from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_analytics_timeseries_body_series_item_filters_annotations_has_annotation_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_annotations_has_annotation_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_guardrails_only_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_label_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_label_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_passed_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_passed_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_score_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_score_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_state_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_state_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_details_key_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_details_key_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_type_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_type_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_key_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_key_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_value_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_value_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_customer_id_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_customer_id_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_key_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_key_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_labels_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_labels_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_prompt_ids_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_prompt_ids_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_thread_id_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_thread_id_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_user_id_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_user_id_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_value_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_value_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_sentiment_input_sentiment_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_sentiment_input_sentiment_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_model_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_model_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_type_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_type_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_subtopics_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_subtopics_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_topics_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_topics_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType2,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_traces_error_type_1 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1,
    )
    from ..models.post_api_analytics_timeseries_body_series_item_filters_traces_error_type_2 import (
        PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType2,
    )


T = TypeVar("T", bound="PostApiAnalyticsTimeseriesBodySeriesItemFilters")


@_attrs_define
class PostApiAnalyticsTimeseriesBodySeriesItemFilters:
    """
    Attributes:
        topics_topics (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType2 | Unset):
        topics_subtopics (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType2 | Unset):
        metadata_user_id (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType2 | Unset):
        metadata_thread_id (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType2 | Unset):
        metadata_customer_id (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType2 | Unset):
        metadata_labels (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType2 | Unset):
        metadata_key (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType2 | Unset):
        metadata_value (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType2 | Unset):
        metadata_prompt_ids (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType2 | Unset):
        traces_error (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType2 | Unset):
        spans_type (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType2 | Unset):
        spans_model (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType2 | Unset):
        evaluations_evaluator_id (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType2 | Unset):
        evaluations_evaluator_id_guardrails_only (list[str] |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2 | Unset):
        evaluations_passed (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType2 | Unset):
        evaluations_score (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType2 | Unset):
        evaluations_state (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType2 | Unset):
        evaluations_label (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType2 | Unset):
        events_event_type (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType2 | Unset):
        events_metrics_key (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType2 | Unset):
        events_metrics_value (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType2 | Unset):
        events_event_details_key (list[str] | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType2 | Unset):
        annotations_has_annotation (list[str] |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType2 | Unset):
        sentiment_input_sentiment (list[str] |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1 |
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType2 | Unset):
    """

    topics_topics: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType2
        | Unset
    ) = UNSET
    topics_subtopics: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType2
        | Unset
    ) = UNSET
    metadata_user_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType2
        | Unset
    ) = UNSET
    metadata_thread_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType2
        | Unset
    ) = UNSET
    metadata_customer_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType2
        | Unset
    ) = UNSET
    metadata_labels: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType2
        | Unset
    ) = UNSET
    metadata_key: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType2
        | Unset
    ) = UNSET
    metadata_value: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType2
        | Unset
    ) = UNSET
    metadata_prompt_ids: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType2
        | Unset
    ) = UNSET
    traces_error: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType2
        | Unset
    ) = UNSET
    spans_type: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType2
        | Unset
    ) = UNSET
    spans_model: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType2
        | Unset
    ) = UNSET
    evaluations_evaluator_id: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType2
        | Unset
    ) = UNSET
    evaluations_evaluator_id_guardrails_only: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2
        | Unset
    ) = UNSET
    evaluations_passed: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType2
        | Unset
    ) = UNSET
    evaluations_score: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType2
        | Unset
    ) = UNSET
    evaluations_state: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType2
        | Unset
    ) = UNSET
    evaluations_label: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType2
        | Unset
    ) = UNSET
    events_event_type: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType2
        | Unset
    ) = UNSET
    events_metrics_key: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType2
        | Unset
    ) = UNSET
    events_metrics_value: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType2
        | Unset
    ) = UNSET
    events_event_details_key: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType2
        | Unset
    ) = UNSET
    annotations_has_annotation: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType2
        | Unset
    ) = UNSET
    sentiment_input_sentiment: (
        list[str]
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1
        | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType2
        | Unset
    ) = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.post_api_analytics_timeseries_body_series_item_filters_annotations_has_annotation_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_label_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_passed_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_score_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_state_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_details_key_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_type_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_key_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_value_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_customer_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_key_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_labels_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_prompt_ids_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_thread_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_user_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_value_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_sentiment_input_sentiment_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_model_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_type_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_subtopics_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_topics_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_traces_error_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1,
        )

        topics_topics: dict[str, Any] | list[str] | Unset
        if isinstance(self.topics_topics, Unset):
            topics_topics = UNSET
        elif isinstance(self.topics_topics, list):
            topics_topics = self.topics_topics

        elif isinstance(self.topics_topics, PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1):
            topics_topics = self.topics_topics.to_dict()
        else:
            topics_topics = self.topics_topics.to_dict()

        topics_subtopics: dict[str, Any] | list[str] | Unset
        if isinstance(self.topics_subtopics, Unset):
            topics_subtopics = UNSET
        elif isinstance(self.topics_subtopics, list):
            topics_subtopics = self.topics_subtopics

        elif isinstance(self.topics_subtopics, PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1):
            topics_subtopics = self.topics_subtopics.to_dict()
        else:
            topics_subtopics = self.topics_subtopics.to_dict()

        metadata_user_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_user_id, Unset):
            metadata_user_id = UNSET
        elif isinstance(self.metadata_user_id, list):
            metadata_user_id = self.metadata_user_id

        elif isinstance(self.metadata_user_id, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1):
            metadata_user_id = self.metadata_user_id.to_dict()
        else:
            metadata_user_id = self.metadata_user_id.to_dict()

        metadata_thread_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_thread_id, Unset):
            metadata_thread_id = UNSET
        elif isinstance(self.metadata_thread_id, list):
            metadata_thread_id = self.metadata_thread_id

        elif isinstance(self.metadata_thread_id, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1):
            metadata_thread_id = self.metadata_thread_id.to_dict()
        else:
            metadata_thread_id = self.metadata_thread_id.to_dict()

        metadata_customer_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_customer_id, Unset):
            metadata_customer_id = UNSET
        elif isinstance(self.metadata_customer_id, list):
            metadata_customer_id = self.metadata_customer_id

        elif isinstance(
            self.metadata_customer_id, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1
        ):
            metadata_customer_id = self.metadata_customer_id.to_dict()
        else:
            metadata_customer_id = self.metadata_customer_id.to_dict()

        metadata_labels: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_labels, Unset):
            metadata_labels = UNSET
        elif isinstance(self.metadata_labels, list):
            metadata_labels = self.metadata_labels

        elif isinstance(self.metadata_labels, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1):
            metadata_labels = self.metadata_labels.to_dict()
        else:
            metadata_labels = self.metadata_labels.to_dict()

        metadata_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_key, Unset):
            metadata_key = UNSET
        elif isinstance(self.metadata_key, list):
            metadata_key = self.metadata_key

        elif isinstance(self.metadata_key, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1):
            metadata_key = self.metadata_key.to_dict()
        else:
            metadata_key = self.metadata_key.to_dict()

        metadata_value: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_value, Unset):
            metadata_value = UNSET
        elif isinstance(self.metadata_value, list):
            metadata_value = self.metadata_value

        elif isinstance(self.metadata_value, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1):
            metadata_value = self.metadata_value.to_dict()
        else:
            metadata_value = self.metadata_value.to_dict()

        metadata_prompt_ids: dict[str, Any] | list[str] | Unset
        if isinstance(self.metadata_prompt_ids, Unset):
            metadata_prompt_ids = UNSET
        elif isinstance(self.metadata_prompt_ids, list):
            metadata_prompt_ids = self.metadata_prompt_ids

        elif isinstance(
            self.metadata_prompt_ids, PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1
        ):
            metadata_prompt_ids = self.metadata_prompt_ids.to_dict()
        else:
            metadata_prompt_ids = self.metadata_prompt_ids.to_dict()

        traces_error: dict[str, Any] | list[str] | Unset
        if isinstance(self.traces_error, Unset):
            traces_error = UNSET
        elif isinstance(self.traces_error, list):
            traces_error = self.traces_error

        elif isinstance(self.traces_error, PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1):
            traces_error = self.traces_error.to_dict()
        else:
            traces_error = self.traces_error.to_dict()

        spans_type: dict[str, Any] | list[str] | Unset
        if isinstance(self.spans_type, Unset):
            spans_type = UNSET
        elif isinstance(self.spans_type, list):
            spans_type = self.spans_type

        elif isinstance(self.spans_type, PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1):
            spans_type = self.spans_type.to_dict()
        else:
            spans_type = self.spans_type.to_dict()

        spans_model: dict[str, Any] | list[str] | Unset
        if isinstance(self.spans_model, Unset):
            spans_model = UNSET
        elif isinstance(self.spans_model, list):
            spans_model = self.spans_model

        elif isinstance(self.spans_model, PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1):
            spans_model = self.spans_model.to_dict()
        else:
            spans_model = self.spans_model.to_dict()

        evaluations_evaluator_id: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_evaluator_id, Unset):
            evaluations_evaluator_id = UNSET
        elif isinstance(self.evaluations_evaluator_id, list):
            evaluations_evaluator_id = self.evaluations_evaluator_id

        elif isinstance(
            self.evaluations_evaluator_id, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1
        ):
            evaluations_evaluator_id = self.evaluations_evaluator_id.to_dict()
        else:
            evaluations_evaluator_id = self.evaluations_evaluator_id.to_dict()

        evaluations_evaluator_id_guardrails_only: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_evaluator_id_guardrails_only, Unset):
            evaluations_evaluator_id_guardrails_only = UNSET
        elif isinstance(self.evaluations_evaluator_id_guardrails_only, list):
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only

        elif isinstance(
            self.evaluations_evaluator_id_guardrails_only,
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        ):
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only.to_dict()
        else:
            evaluations_evaluator_id_guardrails_only = self.evaluations_evaluator_id_guardrails_only.to_dict()

        evaluations_passed: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_passed, Unset):
            evaluations_passed = UNSET
        elif isinstance(self.evaluations_passed, list):
            evaluations_passed = self.evaluations_passed

        elif isinstance(self.evaluations_passed, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1):
            evaluations_passed = self.evaluations_passed.to_dict()
        else:
            evaluations_passed = self.evaluations_passed.to_dict()

        evaluations_score: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_score, Unset):
            evaluations_score = UNSET
        elif isinstance(self.evaluations_score, list):
            evaluations_score = self.evaluations_score

        elif isinstance(self.evaluations_score, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1):
            evaluations_score = self.evaluations_score.to_dict()
        else:
            evaluations_score = self.evaluations_score.to_dict()

        evaluations_state: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_state, Unset):
            evaluations_state = UNSET
        elif isinstance(self.evaluations_state, list):
            evaluations_state = self.evaluations_state

        elif isinstance(self.evaluations_state, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1):
            evaluations_state = self.evaluations_state.to_dict()
        else:
            evaluations_state = self.evaluations_state.to_dict()

        evaluations_label: dict[str, Any] | list[str] | Unset
        if isinstance(self.evaluations_label, Unset):
            evaluations_label = UNSET
        elif isinstance(self.evaluations_label, list):
            evaluations_label = self.evaluations_label

        elif isinstance(self.evaluations_label, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1):
            evaluations_label = self.evaluations_label.to_dict()
        else:
            evaluations_label = self.evaluations_label.to_dict()

        events_event_type: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_event_type, Unset):
            events_event_type = UNSET
        elif isinstance(self.events_event_type, list):
            events_event_type = self.events_event_type

        elif isinstance(self.events_event_type, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1):
            events_event_type = self.events_event_type.to_dict()
        else:
            events_event_type = self.events_event_type.to_dict()

        events_metrics_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_metrics_key, Unset):
            events_metrics_key = UNSET
        elif isinstance(self.events_metrics_key, list):
            events_metrics_key = self.events_metrics_key

        elif isinstance(self.events_metrics_key, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1):
            events_metrics_key = self.events_metrics_key.to_dict()
        else:
            events_metrics_key = self.events_metrics_key.to_dict()

        events_metrics_value: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_metrics_value, Unset):
            events_metrics_value = UNSET
        elif isinstance(self.events_metrics_value, list):
            events_metrics_value = self.events_metrics_value

        elif isinstance(
            self.events_metrics_value, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1
        ):
            events_metrics_value = self.events_metrics_value.to_dict()
        else:
            events_metrics_value = self.events_metrics_value.to_dict()

        events_event_details_key: dict[str, Any] | list[str] | Unset
        if isinstance(self.events_event_details_key, Unset):
            events_event_details_key = UNSET
        elif isinstance(self.events_event_details_key, list):
            events_event_details_key = self.events_event_details_key

        elif isinstance(
            self.events_event_details_key, PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1
        ):
            events_event_details_key = self.events_event_details_key.to_dict()
        else:
            events_event_details_key = self.events_event_details_key.to_dict()

        annotations_has_annotation: dict[str, Any] | list[str] | Unset
        if isinstance(self.annotations_has_annotation, Unset):
            annotations_has_annotation = UNSET
        elif isinstance(self.annotations_has_annotation, list):
            annotations_has_annotation = self.annotations_has_annotation

        elif isinstance(
            self.annotations_has_annotation,
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1,
        ):
            annotations_has_annotation = self.annotations_has_annotation.to_dict()
        else:
            annotations_has_annotation = self.annotations_has_annotation.to_dict()

        sentiment_input_sentiment: dict[str, Any] | list[str] | Unset
        if isinstance(self.sentiment_input_sentiment, Unset):
            sentiment_input_sentiment = UNSET
        elif isinstance(self.sentiment_input_sentiment, list):
            sentiment_input_sentiment = self.sentiment_input_sentiment

        elif isinstance(
            self.sentiment_input_sentiment, PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1
        ):
            sentiment_input_sentiment = self.sentiment_input_sentiment.to_dict()
        else:
            sentiment_input_sentiment = self.sentiment_input_sentiment.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if topics_topics is not UNSET:
            field_dict["topics.topics"] = topics_topics
        if topics_subtopics is not UNSET:
            field_dict["topics.subtopics"] = topics_subtopics
        if metadata_user_id is not UNSET:
            field_dict["metadata.user_id"] = metadata_user_id
        if metadata_thread_id is not UNSET:
            field_dict["metadata.thread_id"] = metadata_thread_id
        if metadata_customer_id is not UNSET:
            field_dict["metadata.customer_id"] = metadata_customer_id
        if metadata_labels is not UNSET:
            field_dict["metadata.labels"] = metadata_labels
        if metadata_key is not UNSET:
            field_dict["metadata.key"] = metadata_key
        if metadata_value is not UNSET:
            field_dict["metadata.value"] = metadata_value
        if metadata_prompt_ids is not UNSET:
            field_dict["metadata.prompt_ids"] = metadata_prompt_ids
        if traces_error is not UNSET:
            field_dict["traces.error"] = traces_error
        if spans_type is not UNSET:
            field_dict["spans.type"] = spans_type
        if spans_model is not UNSET:
            field_dict["spans.model"] = spans_model
        if evaluations_evaluator_id is not UNSET:
            field_dict["evaluations.evaluator_id"] = evaluations_evaluator_id
        if evaluations_evaluator_id_guardrails_only is not UNSET:
            field_dict["evaluations.evaluator_id.guardrails_only"] = evaluations_evaluator_id_guardrails_only
        if evaluations_passed is not UNSET:
            field_dict["evaluations.passed"] = evaluations_passed
        if evaluations_score is not UNSET:
            field_dict["evaluations.score"] = evaluations_score
        if evaluations_state is not UNSET:
            field_dict["evaluations.state"] = evaluations_state
        if evaluations_label is not UNSET:
            field_dict["evaluations.label"] = evaluations_label
        if events_event_type is not UNSET:
            field_dict["events.event_type"] = events_event_type
        if events_metrics_key is not UNSET:
            field_dict["events.metrics.key"] = events_metrics_key
        if events_metrics_value is not UNSET:
            field_dict["events.metrics.value"] = events_metrics_value
        if events_event_details_key is not UNSET:
            field_dict["events.event_details.key"] = events_event_details_key
        if annotations_has_annotation is not UNSET:
            field_dict["annotations.hasAnnotation"] = annotations_has_annotation
        if sentiment_input_sentiment is not UNSET:
            field_dict["sentiment.input_sentiment"] = sentiment_input_sentiment

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_analytics_timeseries_body_series_item_filters_annotations_has_annotation_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_annotations_has_annotation_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_guardrails_only_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_guardrails_only_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_evaluator_id_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_label_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_label_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_passed_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_passed_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_score_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_score_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_state_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_evaluations_state_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_details_key_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_details_key_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_type_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_event_type_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_key_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_key_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_value_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_events_metrics_value_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_customer_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_customer_id_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_key_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_key_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_labels_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_labels_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_prompt_ids_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_prompt_ids_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_thread_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_thread_id_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_user_id_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_user_id_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_value_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_metadata_value_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_sentiment_input_sentiment_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_sentiment_input_sentiment_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_model_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_model_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_type_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_spans_type_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_subtopics_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_subtopics_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_topics_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_topics_topics_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType2,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_traces_error_type_1 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1,
        )
        from ..models.post_api_analytics_timeseries_body_series_item_filters_traces_error_type_2 import (
            PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType2,
        )

        d = dict(src_dict)

        def _parse_topics_topics(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                topics_topics_type_0 = cast(list[str], data)

                return topics_topics_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                topics_topics_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType1.from_dict(data)

                return topics_topics_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            topics_topics_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsTopicsType2.from_dict(data)

            return topics_topics_type_2

        topics_topics = _parse_topics_topics(d.pop("topics.topics", UNSET))

        def _parse_topics_subtopics(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                topics_subtopics_type_0 = cast(list[str], data)

                return topics_subtopics_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                topics_subtopics_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType1.from_dict(
                    data
                )

                return topics_subtopics_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            topics_subtopics_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersTopicsSubtopicsType2.from_dict(
                data
            )

            return topics_subtopics_type_2

        topics_subtopics = _parse_topics_subtopics(d.pop("topics.subtopics", UNSET))

        def _parse_metadata_user_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_user_id_type_0 = cast(list[str], data)

                return metadata_user_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_user_id_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType1.from_dict(
                    data
                )

                return metadata_user_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_user_id_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataUserIdType2.from_dict(data)

            return metadata_user_id_type_2

        metadata_user_id = _parse_metadata_user_id(d.pop("metadata.user_id", UNSET))

        def _parse_metadata_thread_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_thread_id_type_0 = cast(list[str], data)

                return metadata_thread_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_thread_id_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType1.from_dict(data)
                )

                return metadata_thread_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_thread_id_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataThreadIdType2.from_dict(
                data
            )

            return metadata_thread_id_type_2

        metadata_thread_id = _parse_metadata_thread_id(d.pop("metadata.thread_id", UNSET))

        def _parse_metadata_customer_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_customer_id_type_0 = cast(list[str], data)

                return metadata_customer_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_customer_id_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType1.from_dict(data)
                )

                return metadata_customer_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_customer_id_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataCustomerIdType2.from_dict(data)
            )

            return metadata_customer_id_type_2

        metadata_customer_id = _parse_metadata_customer_id(d.pop("metadata.customer_id", UNSET))

        def _parse_metadata_labels(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_labels_type_0 = cast(list[str], data)

                return metadata_labels_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_labels_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType1.from_dict(
                    data
                )

                return metadata_labels_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_labels_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataLabelsType2.from_dict(data)

            return metadata_labels_type_2

        metadata_labels = _parse_metadata_labels(d.pop("metadata.labels", UNSET))

        def _parse_metadata_key(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_key_type_0 = cast(list[str], data)

                return metadata_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_key_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType1.from_dict(data)

                return metadata_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_key_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataKeyType2.from_dict(data)

            return metadata_key_type_2

        metadata_key = _parse_metadata_key(d.pop("metadata.key", UNSET))

        def _parse_metadata_value(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_value_type_0 = cast(list[str], data)

                return metadata_value_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_value_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType1.from_dict(
                    data
                )

                return metadata_value_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_value_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataValueType2.from_dict(data)

            return metadata_value_type_2

        metadata_value = _parse_metadata_value(d.pop("metadata.value", UNSET))

        def _parse_metadata_prompt_ids(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                metadata_prompt_ids_type_0 = cast(list[str], data)

                return metadata_prompt_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_prompt_ids_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType1.from_dict(data)
                )

                return metadata_prompt_ids_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            metadata_prompt_ids_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersMetadataPromptIdsType2.from_dict(data)
            )

            return metadata_prompt_ids_type_2

        metadata_prompt_ids = _parse_metadata_prompt_ids(d.pop("metadata.prompt_ids", UNSET))

        def _parse_traces_error(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                traces_error_type_0 = cast(list[str], data)

                return traces_error_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                traces_error_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType1.from_dict(data)

                return traces_error_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            traces_error_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersTracesErrorType2.from_dict(data)

            return traces_error_type_2

        traces_error = _parse_traces_error(d.pop("traces.error", UNSET))

        def _parse_spans_type(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spans_type_type_0 = cast(list[str], data)

                return spans_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                spans_type_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType1.from_dict(data)

                return spans_type_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            spans_type_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansTypeType2.from_dict(data)

            return spans_type_type_2

        spans_type = _parse_spans_type(d.pop("spans.type", UNSET))

        def _parse_spans_model(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                spans_model_type_0 = cast(list[str], data)

                return spans_model_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                spans_model_type_1 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType1.from_dict(data)

                return spans_model_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            spans_model_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersSpansModelType2.from_dict(data)

            return spans_model_type_2

        spans_model = _parse_spans_model(d.pop("spans.model", UNSET))

        def _parse_evaluations_evaluator_id(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_evaluator_id_type_0 = cast(list[str], data)

                return evaluations_evaluator_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_evaluator_id_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType1.from_dict(data)
                )

                return evaluations_evaluator_id_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_evaluator_id_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdType2.from_dict(data)
            )

            return evaluations_evaluator_id_type_2

        evaluations_evaluator_id = _parse_evaluations_evaluator_id(d.pop("evaluations.evaluator_id", UNSET))

        def _parse_evaluations_evaluator_id_guardrails_only(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_evaluator_id_guardrails_only_type_0 = cast(list[str], data)

                return evaluations_evaluator_id_guardrails_only_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_evaluator_id_guardrails_only_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType1.from_dict(
                        data
                    )
                )

                return evaluations_evaluator_id_guardrails_only_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_evaluator_id_guardrails_only_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsEvaluatorIdGuardrailsOnlyType2.from_dict(data)
            )

            return evaluations_evaluator_id_guardrails_only_type_2

        evaluations_evaluator_id_guardrails_only = _parse_evaluations_evaluator_id_guardrails_only(
            d.pop("evaluations.evaluator_id.guardrails_only", UNSET)
        )

        def _parse_evaluations_passed(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_passed_type_0 = cast(list[str], data)

                return evaluations_passed_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_passed_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType1.from_dict(data)
                )

                return evaluations_passed_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_passed_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsPassedType2.from_dict(
                data
            )

            return evaluations_passed_type_2

        evaluations_passed = _parse_evaluations_passed(d.pop("evaluations.passed", UNSET))

        def _parse_evaluations_score(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_score_type_0 = cast(list[str], data)

                return evaluations_score_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_score_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType1.from_dict(data)
                )

                return evaluations_score_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_score_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsScoreType2.from_dict(
                data
            )

            return evaluations_score_type_2

        evaluations_score = _parse_evaluations_score(d.pop("evaluations.score", UNSET))

        def _parse_evaluations_state(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_state_type_0 = cast(list[str], data)

                return evaluations_state_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_state_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType1.from_dict(data)
                )

                return evaluations_state_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_state_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsStateType2.from_dict(
                data
            )

            return evaluations_state_type_2

        evaluations_state = _parse_evaluations_state(d.pop("evaluations.state", UNSET))

        def _parse_evaluations_label(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                evaluations_label_type_0 = cast(list[str], data)

                return evaluations_label_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluations_label_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType1.from_dict(data)
                )

                return evaluations_label_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            evaluations_label_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersEvaluationsLabelType2.from_dict(
                data
            )

            return evaluations_label_type_2

        evaluations_label = _parse_evaluations_label(d.pop("evaluations.label", UNSET))

        def _parse_events_event_type(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_event_type_type_0 = cast(list[str], data)

                return events_event_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_event_type_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType1.from_dict(data)
                )

                return events_event_type_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_event_type_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventTypeType2.from_dict(
                data
            )

            return events_event_type_type_2

        events_event_type = _parse_events_event_type(d.pop("events.event_type", UNSET))

        def _parse_events_metrics_key(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_metrics_key_type_0 = cast(list[str], data)

                return events_metrics_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_metrics_key_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType1.from_dict(data)
                )

                return events_metrics_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_metrics_key_type_2 = PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsKeyType2.from_dict(
                data
            )

            return events_metrics_key_type_2

        events_metrics_key = _parse_events_metrics_key(d.pop("events.metrics.key", UNSET))

        def _parse_events_metrics_value(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_metrics_value_type_0 = cast(list[str], data)

                return events_metrics_value_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_metrics_value_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType1.from_dict(data)
                )

                return events_metrics_value_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_metrics_value_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsMetricsValueType2.from_dict(data)
            )

            return events_metrics_value_type_2

        events_metrics_value = _parse_events_metrics_value(d.pop("events.metrics.value", UNSET))

        def _parse_events_event_details_key(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                events_event_details_key_type_0 = cast(list[str], data)

                return events_event_details_key_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_event_details_key_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType1.from_dict(data)
                )

                return events_event_details_key_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            events_event_details_key_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersEventsEventDetailsKeyType2.from_dict(data)
            )

            return events_event_details_key_type_2

        events_event_details_key = _parse_events_event_details_key(d.pop("events.event_details.key", UNSET))

        def _parse_annotations_has_annotation(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                annotations_has_annotation_type_0 = cast(list[str], data)

                return annotations_has_annotation_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                annotations_has_annotation_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType1.from_dict(data)
                )

                return annotations_has_annotation_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            annotations_has_annotation_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersAnnotationsHasAnnotationType2.from_dict(data)
            )

            return annotations_has_annotation_type_2

        annotations_has_annotation = _parse_annotations_has_annotation(d.pop("annotations.hasAnnotation", UNSET))

        def _parse_sentiment_input_sentiment(
            data: object,
        ) -> (
            list[str]
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1
            | PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType2
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                sentiment_input_sentiment_type_0 = cast(list[str], data)

                return sentiment_input_sentiment_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sentiment_input_sentiment_type_1 = (
                    PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType1.from_dict(data)
                )

                return sentiment_input_sentiment_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            sentiment_input_sentiment_type_2 = (
                PostApiAnalyticsTimeseriesBodySeriesItemFiltersSentimentInputSentimentType2.from_dict(data)
            )

            return sentiment_input_sentiment_type_2

        sentiment_input_sentiment = _parse_sentiment_input_sentiment(d.pop("sentiment.input_sentiment", UNSET))

        post_api_analytics_timeseries_body_series_item_filters = cls(
            topics_topics=topics_topics,
            topics_subtopics=topics_subtopics,
            metadata_user_id=metadata_user_id,
            metadata_thread_id=metadata_thread_id,
            metadata_customer_id=metadata_customer_id,
            metadata_labels=metadata_labels,
            metadata_key=metadata_key,
            metadata_value=metadata_value,
            metadata_prompt_ids=metadata_prompt_ids,
            traces_error=traces_error,
            spans_type=spans_type,
            spans_model=spans_model,
            evaluations_evaluator_id=evaluations_evaluator_id,
            evaluations_evaluator_id_guardrails_only=evaluations_evaluator_id_guardrails_only,
            evaluations_passed=evaluations_passed,
            evaluations_score=evaluations_score,
            evaluations_state=evaluations_state,
            evaluations_label=evaluations_label,
            events_event_type=events_event_type,
            events_metrics_key=events_metrics_key,
            events_metrics_value=events_metrics_value,
            events_event_details_key=events_event_details_key,
            annotations_has_annotation=annotations_has_annotation,
            sentiment_input_sentiment=sentiment_input_sentiment,
        )

        return post_api_analytics_timeseries_body_series_item_filters



from enum import Enum


class GetApiTracesByTraceIdLlmMode(str, Enum):
    FALSE = "false"
    TRUE = "true"
    VALUE_2 = "1"
    VALUE_3 = "0"

    def __str__(self) -> str:
        return str(self.value)



"""
API service layer for retrieving LangWatch evaluators via REST API.

Provides read-only access to project evaluators with computed fields.
Uses the generated OpenAPI client for type-safe API communication.
"""
from typing import Dict, List, Any

from langwatch.generated.langwatch_rest_api_client.client import (
    Client as LangWatchRestApiClient,
)
from langwatch.generated.langwatch_rest_api_client.api.default import (
    get_api_evaluators,
    get_api_evaluators_by_id_or_slug,
)
from langwatch.generated.langwatch_rest_api_client.models.get_api_evaluators_response_200_item import (
    GetApiEvaluatorsResponse200Item,
)
from langwatch.generated.langwatch_rest_api_client.models.get_api_evaluators_by_id_or_slug_response_200 import (
    GetApiEvaluatorsByIdOrSlugResponse200,
)

from langwatch.utils.initialization import ensure_setup
from langwatch.state import get_instance
from langwatch.prompts.errors import unwrap_response


def _response_to_dict(item: Any) -> Dict[str, Any]:
    """Convert a generated API response item to a plain dictionary."""
    return item.to_dict()


class EvaluatorApiService:
    """
    API service for retrieving LangWatch evaluators via REST API.

    Provides read-only operations with proper error handling and response unwrapping.
    """

    def __init__(self, rest_api_client: LangWatchRestApiClient):
        """Initialize the evaluator API service with a REST API client."""
        self._client = rest_api_client

    @classmethod
    def from_global(cls) -> "EvaluatorApiService":
        """Create an EvaluatorApiService instance using the global LangWatch configuration."""
        ensure_setup()
        instance = get_instance()
        if instance is None:
            raise RuntimeError(
                "LangWatch client has not been initialized. Call setup() first."
            )
        return cls(instance.rest_api_client)

    def get_all(self) -> List[Dict[str, Any]]:
        """Retrieve all evaluators for the project."""
        resp = get_api_evaluators.sync_detailed(client=self._client)
        items = unwrap_response(
            resp,
            ok_type=list,
            subject="evaluators",
            op="fetch all",
        )
        if items is None:
            raise RuntimeError("Failed to fetch evaluators")
        return [_response_to_dict(item) for item in items]

    def get(self, id_or_slug: str) -> Dict[str, Any]:
        """Retrieve a single evaluator by ID or slug."""
        resp = get_api_evaluators_by_id_or_slug.sync_detailed(
            id_or_slug=id_or_slug,
            client=self._client,
        )
        ok = unwrap_response(
            resp,
            ok_type=GetApiEvaluatorsByIdOrSlugResponse200,
            subject=f'id_or_slug="{id_or_slug}"',
            op="fetch",
        )
        if ok is None:
            raise RuntimeError(
                f"Failed to fetch evaluator with id_or_slug={id_or_slug}"
            )
        return _response_to_dict(ok)




from enum import Enum


class PostApiAnalyticsTimeseriesBodySeriesItemPipelineField(str, Enum):
    CUSTOMER_ID = "customer_id"
    THREAD_ID = "thread_id"
    TRACE_ID = "trace_id"
    USER_ID = "user_id"

    def __str__(self) -> str:
        return str(self.value)



"""
End-to-End Tests for Fetch Policies Examples

These tests verify that the fetch policies examples actually work as advertised
by using real API calls and verifying behavior.

Prerequisites:
- LANGWATCH_API_KEY environment variable must be set
- Node.js/npx must be available for CLI operations
"""

import os
from dotenv import load_dotenv

load_dotenv()

import contextlib
import logging
import subprocess
import tempfile
import time
from pathlib import Path
from unittest.mock import patch
from uuid import uuid4

import httpx
import pytest

import langwatch
from langwatch.prompts.local_loader import LocalPromptLoader
from langwatch.prompts.types import FetchPolicy

logger = logging.getLogger(__name__)


@contextlib.contextmanager
def working_directory(path):
    """Temporarily change the working directory."""
    original_cwd = Path.cwd()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(original_cwd)


CLI_EXECUTABLE = ["npx", "langwatch@latest"]


def run_cli(command, cwd=None):
    """Run a CLI command and return the result."""
    try:
        result = subprocess.run(
            command, cwd=cwd, capture_output=True, text=True, check=True, timeout=30
        )
        return result.stdout
    except subprocess.CalledProcessError as e:
        print(f"CLI command failed: {e}")
        print(f"stdout: {e.stdout}")
        print(f"stderr: {e.stderr}")
        raise


@pytest.fixture
def api_key():
    """Ensure API key is available for e2e tests."""
    key = os.getenv("LANGWATCH_API_KEY")
    if not key:
        pytest.skip("LANGWATCH_API_KEY environment variable not set")
    return key


@pytest.fixture
def temp_workspace():
    """Create a temporary workspace directory for CLI operations."""
    with tempfile.TemporaryDirectory() as temp_dir:
        yield Path(temp_dir)


@contextlib.contextmanager
def http_request_counter():
    """
    Context manager that counts HTTP requests made during execution.

    Yields a dictionary with call counts that gets updated in real-time.
    """
    call_counts = {"count": 0}

    original_request = httpx.Client.request

    def counting_request(self, *args, **kwargs):
        call_counts["count"] += 1
        return original_request(self, *args, **kwargs)

    with patch.object(httpx.Client, "request", counting_request):
        yield call_counts


@pytest.mark.e2e
class TestFetchPoliciesE2E:
    """
    End-to-end tests for fetch policies.

    These tests create real prompts on the server, run the example functions,
    and verify they behave as documented.

    To run these tests:
        export LANGWATCH_API_KEY="your-api-key"
        pytest tests/test_fetch_policies_e2e.py -v
    """

    @pytest.fixture(autouse=True)
    def setup_langwatch(self, api_key):
        """Set up LangWatch for each test."""
        LocalPromptLoader._cached_project_root = None
        LocalPromptLoader._warned_no_prompts_path = False
        langwatch.setup(api_key=api_key)
        yield
        LocalPromptLoader._cached_project_root = None
        LocalPromptLoader._warned_no_prompts_path = False

    def test_materialized_first_prefers_local_with_zero_api_calls(self, temp_workspace):
        """
        Test MATERIALIZED_FIRST policy prefers local prompt without API calls.

        GIVEN a prompt exists locally via CLI setup
        WHEN we retrieve it with default policy (MATERIALIZED_FIRST)
        THEN the system returns the local version without API calls
        """
        with working_directory(temp_workspace):
            # Create a unique prompt handle
            handle = f"e2e-materialized-first-local-{uuid4().hex}"

            # Set up local prompt using CLI
            run_cli([*CLI_EXECUTABLE, "prompt", "init"], temp_workspace)
            run_cli([*CLI_EXECUTABLE, "prompt", "create", handle], temp_workspace)
            run_cli(
                [
                    *CLI_EXECUTABLE,
                    "prompt",
                    "add",
                    handle,
                    f"prompts/{handle}.prompt.yaml",
                ],
                temp_workspace,
            )

            langwatch.setup(debug=True)

            # Run the example with HTTP request counting
            with http_request_counter() as calls:
                prompt = langwatch.prompts.get(
                    handle
                )  # Uses default MATERIALIZED_FIRST

            # Verify the prompt was returned correctly
            assert prompt is not None
            assert prompt.handle == handle

            # Should NOT have made any API calls (local-first behavior)
            assert calls["count"] == 0

    def test_materialized_first_falls_back_to_api_when_local_missing(
        self, temp_workspace
    ):
        """
        Test MATERIALIZED_FIRST policy falls back to API when local missing.

        GIVEN a prompt exists on the server but NOT locally
        WHEN we retrieve it with default policy (MATERIALIZED_FIRST)
        THEN the system falls back to API and returns the server version
        """
        with working_directory(temp_workspace):
            # Create a unique prompt handle
            handle = f"e2e-materialized-first-api-{uuid4().hex}"
            prompt_content = "Hello from MATERIALIZED_FIRST policy test"

            langwatch.setup(debug=True)

            # Create prompt on server
            created_prompt = langwatch.prompts.create(
                handle=handle,
                prompt=prompt_content,
            )

            try:
                # Run the example with HTTP request counting
                with http_request_counter() as calls:
                    prompt = langwatch.prompts.get(
                        handle
                    )  # Uses default MATERIALIZED_FIRST

                # Verify the prompt was returned correctly
                assert prompt is not None
                assert prompt.handle == handle
                assert prompt_content in (prompt.prompt or "")

                # Should have made at least one API call (since no local file exists)
                assert calls["count"] > 0

            finally:
                # Clean up
                try:
                    langwatch.prompts.delete(created_prompt.id)
                except Exception as e:
                    logger.warning(
                        "Failed to delete prompt %s: %s", created_prompt.id, e
                    )

    def test_always_fetch_returns_server_prompt_and_hits_api(self, temp_workspace):
        """
        Test ALWAYS_FETCH policy returns server prompt and calls API.

        GIVEN a prompt exists on the server
        WHEN we retrieve it with ALWAYS_FETCH policy
        THEN the system calls the API and returns the server version
        """
        with working_directory(temp_workspace):
            # Create a unique prompt handle
            handle = f"e2e-always-fetch-{uuid4().hex}"
            prompt_content = "Hello from ALWAYS_FETCH policy test"

            langwatch.setup(debug=True)

            # Create prompt on server
            created_prompt = langwatch.prompts.create(
                handle=handle,
                prompt=prompt_content,
            )

            try:
                # Run the example with HTTP request counting
                with http_request_counter() as calls:
                    prompt = langwatch.prompts.get(
                        handle, fetch_policy=FetchPolicy.ALWAYS_FETCH
                    )

                # Verify the prompt was returned correctly
                assert prompt is not None
                assert prompt.handle == handle
                assert prompt_content in (prompt.prompt or "")

                # Should have made at least one API call
                assert calls["count"] > 0

            finally:
                # Clean up
                try:
                    langwatch.prompts.delete(created_prompt.id)
                except Exception as e:
                    logger.warning(
                        "Failed to delete prompt %s: %s", created_prompt.id, e
                    )

    def test_materialized_only_returns_local_prompt_without_api_call(
        self, temp_workspace
    ):
        """
        Test MATERIALIZED_ONLY policy uses local files without API calls.

        GIVEN a prompt exists locally via CLI setup
        WHEN we retrieve it with MATERIALIZED_ONLY policy
        THEN the system returns the local version without API calls
        """
        with working_directory(temp_workspace):
            # Create a unique prompt handle
            handle = f"e2e-materialized-only-{uuid4().hex}"

            # Setup materialized only prompt using CLI
            run_cli([*CLI_EXECUTABLE, "prompt", "init"], temp_workspace)

            langwatch.setup(debug=True)

            # Create prompt on server
            server_prompt = langwatch.prompts.create(
                handle=handle, prompt="Hello from MATERIALIZED_ONLY policy test"
            )

            print(f"server_prompt: {server_prompt}")

            # CLI add prompt to local project
            add_result = run_cli(
                [*CLI_EXECUTABLE, "prompt", "add", handle],
                temp_workspace,
            )
            print(f"add_result: {add_result}")

            sync_result = run_cli([*CLI_EXECUTABLE, "prompt", "sync"], temp_workspace)
            print(f"sync_result: {sync_result}")

            # Run the example with HTTP request counting
            with http_request_counter() as calls:
                prompt = langwatch.prompts.get(
                    handle, fetch_policy=FetchPolicy.MATERIALIZED_ONLY
                )

            # Verify the prompt was returned correctly
            assert prompt is not None
            assert prompt.handle == handle

            # Should NOT have made any API calls
            assert calls["count"] == 0

    def test_cache_ttl_caches_then_refreshes_after_expiry(self, temp_workspace):
        """
        Test CACHE_TTL policy caches responses and refreshes after TTL expiry.

        GIVEN a prompt exists on the server
        WHEN we retrieve it twice with CACHE_TTL policy using short TTL
        THEN the first call caches and second call refreshes after expiry
        """
        with working_directory(temp_workspace):
            # Create a unique prompt handle
            handle = f"e2e-cache-ttl-{uuid4().hex}"
            prompt_content = "Hello from CACHE_TTL policy test"

            langwatch.setup(debug=True)

            # Create prompt on server
            created_prompt = langwatch.prompts.create(
                handle=handle,
                prompt=prompt_content,
            )

            try:
                # First call - should hit API and cache
                with http_request_counter() as calls_first:
                    prompt1 = langwatch.prompts.get(
                        handle,
                        fetch_policy=FetchPolicy.CACHE_TTL,
                        cache_ttl_minutes=0.0005,
                    )  # Very short TTL

                assert prompt1 is not None
                assert calls_first["count"] > 0

                # Wait for cache to expire
                time.sleep(0.1)  # 0.1 seconds > 0.03 seconds TTL

                # Second call - cache expired, should hit API again
                with http_request_counter() as calls_second:
                    prompt2 = langwatch.prompts.get(
                        handle,
                        fetch_policy=FetchPolicy.CACHE_TTL,
                        cache_ttl_minutes=0.0005,
                    )

                assert prompt2 is not None
                assert calls_second["count"] > 0

                # Both prompts should be the same
                assert prompt1.handle == prompt2.handle
                assert prompt1.id == prompt2.id

            finally:
                # Clean up
                try:
                    langwatch.prompts.delete(created_prompt.id)
                except Exception as e:
                    logger.warning(
                        "Failed to delete prompt %s: %s", created_prompt.id, e
                    )



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_demonstrations import (
        PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataDemonstrations,
    )
    from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_inputs_item import (
        PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataInputsItem,
    )
    from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_messages_item import (
        PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataMessagesItem,
    )
    from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_outputs_item import (
        PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataOutputsItem,
    )
    from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_prompting_technique import (
        PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataPromptingTechnique,
    )
    from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_response_format import (
        PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataResponseFormat,
    )


T = TypeVar("T", bound="PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigData")


@_attrs_define
class PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigData:
    """
    Attributes:
        prompt (str):
        messages (list[PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataMessagesItem]):
        inputs (list[PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataInputsItem]):
        outputs (list[PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataOutputsItem]):
        model (str):
        temperature (float | Unset):
        max_tokens (float | Unset):
        top_p (float | Unset):
        frequency_penalty (float | Unset):
        presence_penalty (float | Unset):
        seed (float | Unset):
        top_k (float | Unset):
        min_p (float | Unset):
        repetition_penalty (float | Unset):
        reasoning (str | Unset):
        reasoning_effort (str | Unset):
        thinking_level (str | Unset):
        effort (str | Unset):
        verbosity (str | Unset):
        demonstrations (PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataDemonstrations | Unset):
        prompting_technique (PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataPromptingTechnique | Unset):
        response_format (PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataResponseFormat | Unset):
    """

    prompt: str
    messages: list[PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataMessagesItem]
    inputs: list[PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataInputsItem]
    outputs: list[PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataOutputsItem]
    model: str
    temperature: float | Unset = UNSET
    max_tokens: float | Unset = UNSET
    top_p: float | Unset = UNSET
    frequency_penalty: float | Unset = UNSET
    presence_penalty: float | Unset = UNSET
    seed: float | Unset = UNSET
    top_k: float | Unset = UNSET
    min_p: float | Unset = UNSET
    repetition_penalty: float | Unset = UNSET
    reasoning: str | Unset = UNSET
    reasoning_effort: str | Unset = UNSET
    thinking_level: str | Unset = UNSET
    effort: str | Unset = UNSET
    verbosity: str | Unset = UNSET
    demonstrations: PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataDemonstrations | Unset = UNSET
    prompting_technique: PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataPromptingTechnique | Unset = UNSET
    response_format: PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataResponseFormat | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        prompt = self.prompt

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        inputs = []
        for inputs_item_data in self.inputs:
            inputs_item = inputs_item_data.to_dict()
            inputs.append(inputs_item)

        outputs = []
        for outputs_item_data in self.outputs:
            outputs_item = outputs_item_data.to_dict()
            outputs.append(outputs_item)

        model = self.model

        temperature = self.temperature

        max_tokens = self.max_tokens

        top_p = self.top_p

        frequency_penalty = self.frequency_penalty

        presence_penalty = self.presence_penalty

        seed = self.seed

        top_k = self.top_k

        min_p = self.min_p

        repetition_penalty = self.repetition_penalty

        reasoning = self.reasoning

        reasoning_effort = self.reasoning_effort

        thinking_level = self.thinking_level

        effort = self.effort

        verbosity = self.verbosity

        demonstrations: dict[str, Any] | Unset = UNSET
        if not isinstance(self.demonstrations, Unset):
            demonstrations = self.demonstrations.to_dict()

        prompting_technique: dict[str, Any] | Unset = UNSET
        if not isinstance(self.prompting_technique, Unset):
            prompting_technique = self.prompting_technique.to_dict()

        response_format: dict[str, Any] | Unset = UNSET
        if not isinstance(self.response_format, Unset):
            response_format = self.response_format.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prompt": prompt,
                "messages": messages,
                "inputs": inputs,
                "outputs": outputs,
                "model": model,
            }
        )
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if top_p is not UNSET:
            field_dict["top_p"] = top_p
        if frequency_penalty is not UNSET:
            field_dict["frequency_penalty"] = frequency_penalty
        if presence_penalty is not UNSET:
            field_dict["presence_penalty"] = presence_penalty
        if seed is not UNSET:
            field_dict["seed"] = seed
        if top_k is not UNSET:
            field_dict["top_k"] = top_k
        if min_p is not UNSET:
            field_dict["min_p"] = min_p
        if repetition_penalty is not UNSET:
            field_dict["repetition_penalty"] = repetition_penalty
        if reasoning is not UNSET:
            field_dict["reasoning"] = reasoning
        if reasoning_effort is not UNSET:
            field_dict["reasoning_effort"] = reasoning_effort
        if thinking_level is not UNSET:
            field_dict["thinkingLevel"] = thinking_level
        if effort is not UNSET:
            field_dict["effort"] = effort
        if verbosity is not UNSET:
            field_dict["verbosity"] = verbosity
        if demonstrations is not UNSET:
            field_dict["demonstrations"] = demonstrations
        if prompting_technique is not UNSET:
            field_dict["prompting_technique"] = prompting_technique
        if response_format is not UNSET:
            field_dict["response_format"] = response_format

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_demonstrations import (
            PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataDemonstrations,
        )
        from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_inputs_item import (
            PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataInputsItem,
        )
        from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_messages_item import (
            PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataMessagesItem,
        )
        from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_outputs_item import (
            PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataOutputsItem,
        )
        from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_prompting_technique import (
            PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataPromptingTechnique,
        )
        from ..models.post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data_response_format import (
            PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataResponseFormat,
        )

        d = dict(src_dict)
        prompt = d.pop("prompt")

        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataMessagesItem.from_dict(
                messages_item_data
            )

            messages.append(messages_item)

        inputs = []
        _inputs = d.pop("inputs")
        for inputs_item_data in _inputs:
            inputs_item = PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataInputsItem.from_dict(
                inputs_item_data
            )

            inputs.append(inputs_item)

        outputs = []
        _outputs = d.pop("outputs")
        for outputs_item_data in _outputs:
            outputs_item = PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataOutputsItem.from_dict(
                outputs_item_data
            )

            outputs.append(outputs_item)

        model = d.pop("model")

        temperature = d.pop("temperature", UNSET)

        max_tokens = d.pop("max_tokens", UNSET)

        top_p = d.pop("top_p", UNSET)

        frequency_penalty = d.pop("frequency_penalty", UNSET)

        presence_penalty = d.pop("presence_penalty", UNSET)

        seed = d.pop("seed", UNSET)

        top_k = d.pop("top_k", UNSET)

        min_p = d.pop("min_p", UNSET)

        repetition_penalty = d.pop("repetition_penalty", UNSET)

        reasoning = d.pop("reasoning", UNSET)

        reasoning_effort = d.pop("reasoning_effort", UNSET)

        thinking_level = d.pop("thinkingLevel", UNSET)

        effort = d.pop("effort", UNSET)

        verbosity = d.pop("verbosity", UNSET)

        _demonstrations = d.pop("demonstrations", UNSET)
        demonstrations: PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataDemonstrations | Unset
        if isinstance(_demonstrations, Unset):
            demonstrations = UNSET
        else:
            demonstrations = PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataDemonstrations.from_dict(
                _demonstrations
            )

        _prompting_technique = d.pop("prompting_technique", UNSET)
        prompting_technique: PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataPromptingTechnique | Unset
        if isinstance(_prompting_technique, Unset):
            prompting_technique = UNSET
        else:
            prompting_technique = (
                PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataPromptingTechnique.from_dict(
                    _prompting_technique
                )
            )

        _response_format = d.pop("response_format", UNSET)
        response_format: PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataResponseFormat | Unset
        if isinstance(_response_format, Unset):
            response_format = UNSET
        else:
            response_format = PostApiPromptsByIdSyncResponse200ConflictInfoRemoteConfigDataResponseFormat.from_dict(
                _response_format
            )

        post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data = cls(
            prompt=prompt,
            messages=messages,
            inputs=inputs,
            outputs=outputs,
            model=model,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
            frequency_penalty=frequency_penalty,
            presence_penalty=presence_penalty,
            seed=seed,
            top_k=top_k,
            min_p=min_p,
            repetition_penalty=repetition_penalty,
            reasoning=reasoning,
            reasoning_effort=reasoning_effort,
            thinking_level=thinking_level,
            effort=effort,
            verbosity=verbosity,
            demonstrations=demonstrations,
            prompting_technique=prompting_technique,
            response_format=response_format,
        )

        post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data.additional_properties = d
        return post_api_prompts_by_id_sync_response_200_conflict_info_remote_config_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# src/langwatch/prompts/prompt_api_service.py
"""
API service layer for managing LangWatch prompts via REST API.

This module provides a focused interface for CRUD operations on prompts via API only,
handling API communication, error handling, and response unwrapping.
Uses TypedDict for clean interfaces and from_dict methods for type safety.

This service is responsible only for API operations and does not handle local file loading.
"""
from typing import Dict, List, Literal, Optional
from langwatch.generated.langwatch_rest_api_client.types import UNSET
from langwatch.generated.langwatch_rest_api_client.client import (
    Client as LangWatchRestApiClient,
)
from langwatch.generated.langwatch_rest_api_client.api.default import (
    get_api_prompts_by_id,
    post_api_prompts,
    put_api_prompts_by_id,
    delete_api_prompts_by_id,
)
from langwatch.generated.langwatch_rest_api_client.models.get_api_prompts_by_id_response_200 import (
    GetApiPromptsByIdResponse200,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_body import (
    PostApiPromptsBody,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_body_messages_item import (
    PostApiPromptsBodyMessagesItem,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_body_inputs_item import (
    PostApiPromptsBodyInputsItem,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_body_outputs_item import (
    PostApiPromptsBodyOutputsItem,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_body_scope import (
    PostApiPromptsBodyScope,
)
from langwatch.generated.langwatch_rest_api_client.models.post_api_prompts_response_200 import (
    PostApiPromptsResponse200,
)
from langwatch.generated.langwatch_rest_api_client.models.put_api_prompts_by_id_body import (
    PutApiPromptsByIdBody,
    PutApiPromptsByIdBodyScope,
)
from langwatch.generated.langwatch_rest_api_client.models.put_api_prompts_by_id_body_messages_item import (
    PutApiPromptsByIdBodyMessagesItem,
)
from langwatch.generated.langwatch_rest_api_client.models.put_api_prompts_by_id_body_inputs_item import (
    PutApiPromptsByIdBodyInputsItem,
)
from langwatch.generated.langwatch_rest_api_client.models.put_api_prompts_by_id_body_outputs_item import (
    PutApiPromptsByIdBodyOutputsItem,
)
from langwatch.generated.langwatch_rest_api_client.models.put_api_prompts_by_id_response_200 import (
    PutApiPromptsByIdResponse200,
)
from langwatch.generated.langwatch_rest_api_client.models.delete_api_prompts_by_id_response_200 import (
    DeleteApiPromptsByIdResponse200,
)

from langwatch.utils.initialization import ensure_setup
from langwatch.state import get_instance
from .errors import unwrap_response
from .decorators.prompt_service_tracing import prompt_service_tracing
from .types import PromptData, Message, Input, Output, MessageDict, InputDict, OutputDict


class PromptApiService:
    """
    API service for managing LangWatch prompts via REST API only.

    Provides CRUD operations for prompts with proper error handling and response
    unwrapping. Uses TypedDict interfaces for clean, type-safe API.

    This service handles only API operations and does not handle local file loading.
    """

    def __init__(self, rest_api_client: LangWatchRestApiClient):
        """Initialize the prompt API service with a REST API client."""
        self._client = rest_api_client

    @classmethod
    def from_global(cls) -> "PromptApiService":
        """Create a PromptApiService instance using the global LangWatch configuration."""
        ensure_setup()
        instance = get_instance()
        if instance is None:
            raise RuntimeError(
                "LangWatch client has not been initialized. Call setup() first."
            )
        return cls(instance.rest_api_client)

    @prompt_service_tracing.get
    def get(self, prompt_id: str, version_number: Optional[int] = None) -> PromptData:
        """Retrieve a prompt by its ID from the API. You can optionally specify a version number to get a specific version of the prompt."""
        resp = get_api_prompts_by_id.sync_detailed(
            id=prompt_id,
            client=self._client,
            version=version_number if version_number is not None else UNSET,
        )
        ok = unwrap_response(
            resp,
            ok_type=GetApiPromptsByIdResponse200,
            subject=f'handle_or_id="{prompt_id}"',
            op="fetch",
        )
        if ok is None:
            raise RuntimeError(
                f"Failed to fetch prompt with handle_or_id={prompt_id} version={version_number if version_number is not None else 'latest'}"
            )
        return PromptData.from_api_response(ok)

    def create(
        self,
        handle: str,
        author_id: Optional[str] = None,
        scope: Literal["PROJECT", "ORGANIZATION"] = "PROJECT",
        prompt: Optional[str] = None,
        messages: Optional[List[MessageDict]] = None,
        inputs: Optional[List[InputDict]] = None,
        outputs: Optional[List[OutputDict]] = None,
    ) -> PromptData:
        """
        Create a new prompt with clean dictionary interfaces.

        Args:
            handle: Unique identifier for the prompt
            author_id: ID of the author
            scope: Scope of the prompt ('PROJECT' or 'ORGANIZATION')
            prompt: The prompt text content
            messages: List of message dicts with 'role' and 'content' keys
            inputs: List of input dicts with 'identifier' and 'type' keys
            outputs: List of output dicts with 'identifier', 'type', and optional 'json_schema' keys

        Returns:
            PromptData dictionary containing the created prompt data
        """
        api_messages = UNSET
        if messages:
            api_messages = [
                PostApiPromptsBodyMessagesItem.from_dict(msg.model_dump())
                for msg in messages
            ]

        api_inputs = UNSET
        if inputs:
            api_inputs = [
                PostApiPromptsBodyInputsItem.from_dict(inp.model_dump())
                for inp in inputs
            ]

        api_outputs = UNSET
        if outputs:
            api_outputs = [
                PostApiPromptsBodyOutputsItem.from_dict(out.model_dump())
                for out in outputs
            ]

        resp = post_api_prompts.sync_detailed(
            client=self._client,
            body=PostApiPromptsBody(
                handle=handle,
                scope=PostApiPromptsBodyScope(scope),
                author_id=author_id or UNSET,
                prompt=prompt or UNSET,
                messages=api_messages,
                inputs=api_inputs,
                outputs=api_outputs,
            ),
        )
        ok = unwrap_response(
            resp,
            ok_type=PostApiPromptsResponse200,
            subject=f'handle="{handle}"',
            op="create",
        )
        if ok is None:
            raise RuntimeError(f"Failed to create prompt with handle={handle}")

        return PromptData.from_api_response(ok)

    def update(
        self,
        prompt_id_or_handle: str,
        scope: Literal["PROJECT", "ORGANIZATION"],
        handle: Optional[str] = None,
        prompt: Optional[str] = None,
        messages: Optional[List[MessageDict]] = None,
        inputs: Optional[List[InputDict]] = None,
        outputs: Optional[List[OutputDict]] = None,
    ) -> PromptData:
        """
        Update an existing prompt with clean dictionary interfaces.

        Args:
            prompt_id_or_handle: ID or handle of the prompt to update
            scope: Scope of the prompt
            handle: New handle for the prompt
            prompt: New prompt text content
            messages: New list of message dicts
            inputs: New list of input dicts
            outputs: New list of output dicts

        Returns:
            PromptData dictionary containing the updated prompt data
        """
        api_messages = UNSET
        if messages:
            api_messages = [
                PutApiPromptsByIdBodyMessagesItem.from_dict(msg.model_dump())
                for msg in messages
            ]

        api_inputs = UNSET
        if inputs:
            api_inputs = [
                PutApiPromptsByIdBodyInputsItem.from_dict(inp.model_dump())
                for inp in inputs
            ]

        api_outputs = UNSET
        if outputs:
            api_outputs = [
                PutApiPromptsByIdBodyOutputsItem.from_dict(out.model_dump())
                for out in outputs
            ]

        resp = put_api_prompts_by_id.sync_detailed(
            id=prompt_id_or_handle,
            client=self._client,
            body=PutApiPromptsByIdBody(
                handle=handle or UNSET,
                scope=PutApiPromptsByIdBodyScope[scope],
                prompt=prompt or UNSET,
                messages=api_messages,
                inputs=api_inputs,
                outputs=api_outputs,
            ),
        )

        ok = unwrap_response(
            resp,
            ok_type=PutApiPromptsByIdResponse200,
            subject=f'id="{prompt_id_or_handle}"',
            op="update",
        )
        if ok is None:
            raise RuntimeError(f"Failed to update prompt with id={prompt_id_or_handle}")

        return PromptData.from_api_response(ok)

    def delete(self, prompt_id: str) -> Dict[str, bool]:
        """Delete a prompt by its ID."""
        resp = delete_api_prompts_by_id.sync_detailed(id=prompt_id, client=self._client)
        ok = unwrap_response(
            resp,
            ok_type=DeleteApiPromptsByIdResponse200,
            subject=f'id="{prompt_id}"',
            op="delete",
        )
        if ok is None:
            raise RuntimeError(f"Failed to delete prompt with id={prompt_id}")
        return {"success": bool(ok.success)}
